﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.BisnessLogicsContracts;
using ComputersShopDataModels.Enums;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ComputersShop
{
	public partial class FormMain : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;
		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;

		public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
		}

		private void ComponentsStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormComponents));
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}
		}

		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void LoadData()
		{
			_logger.LogInformation("Загрузка заказов");
			try
			{
				var list = _orderLogic.ReadList(null);
				if (list != null)
				{
					dataGridView.DataSource = list;
					dataGridView.Columns["ComputerId"].Visible = false;
					dataGridView.Columns["ClientId"].Visible = false;
					dataGridView.Columns["ImplementerId"].Visible = false;
					dataGridView.Columns["ComputerName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
				}
				_logger.LogInformation("Загрузка заказов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки заказов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ComputersStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormComputers));
			if (service is FormComputers form)
			{
				form.ShowDialog();
			}
		}

		private void CreateOrderButton_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}

		private void списокКомпонентовToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveComponentsToWordFile(new ReportBindingModel
				{
					FileName = dialog.FileName
				});
				MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK,
			   MessageBoxIcon.Information);
			}

		}

		private void компонентыПоКомпьютерамToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportComputerComponents));
			if (service is FormReportComputerComponents form)
			{
				form.ShowDialog();
			}

		}

		private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
			if (service is FormReportOrders form)
			{
				form.ShowDialog();
			}

		}

		private void клиентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormClients));
			if (service is FormClients form)
			{
				form.ShowDialog();
			}
		}

		private void исполнителиToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(ImplementersForm));
			if (service is ImplementersForm form)
			{
				form.ShowDialog();
			}
		}

		private void запускРаботToolStripMenuItem_Click(object sender, EventArgs e)
		{

			_workProcess.DoWork((Program.ServiceProvider?.GetService(typeof(IImplementerLogic
			)) as IImplementerLogic)!, _orderLogic);
			MessageBox.Show("Процесс обработки запущен", "Сообщение",
		   MessageBoxButtons.OK, MessageBoxIcon.Information);

		}

		private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
		{
			return new OrderBindingModel
			{
				Id = id,
				ComputerId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["ComputerId"].Value),
				ClientId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["ClientId"].Value),
				Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
				Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
				Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
				DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
			};
		}

		private void TakeInWorkButton_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
				try
				{
					var operationResult = _orderLogic.TakeOrderInWork(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка передачи заказа в работу");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}

		}

		private void ReadyButton_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'", id);
				try
				{
					var operationResult = _orderLogic.FinishOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о готовности заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void IssuedButton_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					_logger.LogInformation("Заказ №{id} выдан", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о выдачи заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}

		}

		private void RefreshButton_Click(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}
