﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels.Enums;

using System.ComponentModel.DataAnnotations;

namespace ComputersShopDatabaseImplement.Models
{
    public class Order
    {
        public int Id { get; private set; }
        [Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; }
        [Required]
        public DateTime DateCreate { get; private set; }
        public DateTime? DateImplement { get; private set; }
        [Required]
        public int ComputerId { get; private set; }
        public virtual Computer? Computers { get; private set; }
        [Required]
        public int ClientId { get; private set; }
        public virtual Client? Client { get; private set; }
		public int? ImplementerId { get; private set; } = null;
		public virtual Implementer? Implementer { get; private set; }
		public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ComputerId = model.ComputerId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            ComputerId = ComputerId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            Id = Id,
            ComputerName = Computers?.ComputerName ?? string.Empty,
            ClientFIO = Client?.ClientFIO ?? string.Empty,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
        };
    }
}
