﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels;
using ComputersShopDataModels.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ComputersShopFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int ComputerId { get; private set; }
        public int ClientId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; }
        public DateTime DateCreate { get; private set; }
        public DateTime? DateImplement { get; private set; }
		public int? ImplementerId { get; private set; } = null;

		public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ComputerId = model.ComputerId,
                ClientId = model.ClientId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ImplementerId = model.ImplementerId,
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ComputerId = Convert.ToInt32(element.Element("ComputerId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
				ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value.ToString()),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                DateImplement = string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? null : Convert.ToDateTime(element.Element("DateImplement")!.Value)
            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ComputerId = ComputerId,
            ClientId = ClientId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ImplementerId = ImplementerId
        };
        public XElement GetXElement => new("Order",
        new XAttribute("Id", Id),
        new XElement("ComputerId", ComputerId),
        new XElement("ClientId", ClientId),
        new XElement("Sum", Sum.ToString()),
        new XElement("Count", Count),
        new XElement("Status", Status.ToString()),
        new XElement("DateCreate", DateCreate.ToString()),
        new XElement("DateImplement", DateImplement.ToString()),
        new XElement("ImplementerId", ImplementerId)
        );
    }
}
